Class = require "hump.class"
vector = require "hump.vector"
PrettyPrint = require "libs.pp.PrettyPrint"

Actor = Class{
    init = function(self, pos, r, level)
        self.pos = pos
        self.level = level or 5
        self.r = function(self)
            return r + self.level*2
        end

        self.speed = 100
        self.foodtimer = Timer.new()

        local rand = love.math.random(-1,1 )
        self.foodtimer:addPeriodic(Actor.eatCycle/self.level + rand, function() self:eat() end)

        self.rawR = r

        self.right = anim8.newAnimation(spriteGrid('1-2',self.level), 0.1)
        self.left = anim8.newAnimation(spriteGrid('5-6',self.level), 0.1)
        self.up = anim8.newAnimation(spriteGrid('3-4',self.level), 0.1)
        self.down = anim8.newAnimation(spriteGrid('7-8',self.level), 0.1)
    end,
    levelCap = 8,
    convertSpeed = 1,
    eatCycle = 50
}

function Actor:update(dt, player)
    self.right:update(dt)
    self.left:update(dt)
    self.up:update(dt)
    self.down:update(dt)

    self.foodtimer:update(dt)

    for i, actor in ipairs(self.group.member) do
        if self.pos:dist(actor.pos) < self:r() + actor:r() + 10 then
            local dv = (self.pos - actor.pos) * 3
            actor.pos = actor.pos - (dv*dt)
            self.pos = self.pos + (dv*dt)
        end
    end

    for i, obstacle in ipairs(obstacles) do
        if collide(self.pos,self:r(), obstacle.pos, obstacle.r) then
            local back = (self.pos - obstacle.pos)*10
            local side = back:rotated(90)
            self.pos = self.pos + ((back+side)*dt)
        end
    end

    for i, obstacle in ipairs(allFoodPlaces) do
        if collide(self.pos,self:r(), obstacle.pos, 40) then
            local back = (self.pos - obstacle.pos)*10
            local side = back:rotated(90)
            self.pos = self.pos + ((back+side)*dt)
        end
    end

    if self.goal then
        self:step(self.goal,dt)
    end
end

function Actor:draw()
    love.graphics.circle( "fill", self.pos.x, self.pos.y, self:r(), 100 )

    love.graphics.setColor( 0, 0, 0, 255 )
    love.graphics.print( self.level, self.pos.x, self.pos.y, 0, self.level, self.level)
    love.graphics.setColor( 255, 255, 255, 255 )
end

function Actor:step(vec,dt)
    local d = vec - self.pos
    --d.x = math.clamp(-self.speed,d.x,self.speed)
    --d.y = math.clamp(-self.speed,d.y,self.speed)
    if d:len() > 100 then
        self.pos = self.pos + (d*dt)
    end
end

function Actor:eat()
    local n = self:needs()
    --print(n, self.group.food)
    if self.group.food >= n then
        self:grow()
        self.group.food = self.group.food - n
    else
        self:shrink()
    end
end

function Actor:grow(n)
    local n = n or 1
    if self.level + n <= Actor.levelCap then
        self.level = self.level + n

        love.audio.play( eatS )
    end

    self.foodtimer:clear()
    self.foodtimer:addPeriodic(Actor.eatCycle/self.level, function() self:eat() end)

    self.right = anim8.newAnimation(spriteGrid('1-2',self.level), 0.1)
    self.left = anim8.newAnimation(spriteGrid('5-6',self.level), 0.1)
    self.up = anim8.newAnimation(spriteGrid('3-4',self.level), 0.1)
    self.down = anim8.newAnimation(spriteGrid('7-8',self.level), 0.1)

end

function Actor:shrink(n)
    local n = n or 1
    self.level = self.level - n

    if self.level < 1 then
        self.level = 0
        self:die()
    else
        love.audio.play( leveldownS )

        self.right = anim8.newAnimation(spriteGrid('1-2',self.level), 0.1)
        self.left = anim8.newAnimation(spriteGrid('5-6',self.level), 0.1)
        self.up = anim8.newAnimation(spriteGrid('3-4',self.level), 0.1)
        self.down = anim8.newAnimation(spriteGrid('7-8',self.level), 0.1)
    end
end

function Actor:die()
    love.audio.play( dieS )

    self.foodtimer:clear()
    self.group:removeMember(self)
end

function Actor:needs()
    return self.level
end

function Actor:nearActors(range)
    local inRange = {}
    for i, actor in ipairs(self.group.member) do
        if self.pos:dist(actor.pos) < self:r() + actor:r() + range then
            table.insert(inRange, actor)
        end
    end
    return inRange
end

return Actor