Class = require "hump.class"
vector = require "hump.vector"
PrettyPrint = require "libs.pp.PrettyPrint"
Actor = require "actor"

Enemy = Class{
    init = function(self, pos, r, level)
                self.pos = pos
                self.r = function() return r end
                self.range = 450
                self.attackRange = 100
                self.level = level or 10
                self.timer = Timer.new()
                self.timer:addPeriodic(1, function() self:fight(self:nextActor()) end)
                self.speed = 90
                self.anim = anim8.newAnimation(enemyGrid('1-2',1), 0.1)
                self.hitAnim = anim8.newAnimation(hitGrid('1-2',1), 0.2)
           end,
    color = "evil"
}
Enemy:include(Actor)

function Enemy:update(dt)
    self.timer:update(dt)
    self.anim:update(dt)
    self.hitAnim:update(dt)

    for i, actor in ipairs(playerGroup.member) do
        if self.pos:dist(actor.pos) < self:r() + actor:r() + 10 then
            local dv = self.pos - actor.pos
            actor.pos = actor.pos - (dv*dt)
            self.pos = self.pos + (dv*dt)
        end
    end

    for i, obstacle in ipairs(obstacles) do
        if collide(self.pos,self:r(), obstacle.pos, obstacle.r) then
            local back = (self.pos - obstacle.pos)*2
            local side = back:rotated(90)
            self.pos = self.pos + ((back+side)*dt)
        end
    end

    local m = playerGroup:middle()
    if self.pos:dist(m) < self.range then
        self:step(playerGroup:middle(),dt)
    end
end

function Enemy:draw()
    --love.graphics.draw(enemySprite, self.pos.x - enemySprite:getWidth()/2, self.pos.y - enemySprite:getHeight()/2)
    self.anim:draw(enemySprite, self.pos.x - enemySprite:getWidth()/2, self.pos.y - enemySprite:getHeight()/2, 0, 3, 3)
    --self.hitAnim:draw(hitSprite,self.pos.x, self.pos.y)
end

function Enemy:nextActor()
    local withD = _.map(playerGroup.member, function(actor)
        local d = self.pos:dist(actor.pos)
        if d <= self.attackRange then
            return {actor = actor, d = d}
        end
    end)

    local minWithD = _.min(withD,function(t)
        return t.d
    end)

    if minWithD then return minWithD.actor else return nil end
end

function Enemy:fight(actor)
    if actor then
        actor:shrink(1)
        drawTimer:do_for(0.4, function()
            love.graphics.setColor( 255, 50, 50, 255 )
            self.hitAnim:draw(hitSprite,actor.pos.x - 32, actor.pos.y - 20)
            love.graphics.setColor( 255, 255, 255, 255 )
        end)
    end
end

function Enemy:die()
    for i, v in ipairs(enemies) do
        if v == self then
            --Timer.cancel(self.handle)       -- FIXMEE: doesn't do anything :(
            self.timer:clear()  --sad fix
            self.level = 0
            table.remove(enemies, i)
        end
    end
end

function Enemy:step(vec, dt)
    local d = vec - self.pos
    d.x = math.clamp(-self.speed,d.x,self.speed)
    d.y = math.clamp(-self.speed,d.y,self.speed)
    --if d:len() > 100 then
        self.pos = self.pos + (d*dt)
    --end
end

function Enemy:shrink(n)
    local n = n or 1
    self.level = self.level - n

    love.audio.play( hitS )

    if self.level < 1 then
        self.level = 0
        self:die()
    end
end

return Enemy