require "util"
Timer = require "hump.timer"
vector = require "hump.vector"
Actor = require "actor"
Player = require "player"
Red = require "red"
Blue = require "blue"
Group = require "group"
FoodPlace = require "foodplace"
local loader = require "AdvTiledLoader.Loader"
PrettyPrint = require "libs.pp.PrettyPrint"
Object = require "object"
require "libs.generalMath"
Enemy = require "enemy"
_ = require "libs.underscore"
Camera = require "hump.camera"
anim8 = require 'anim8.anim8'
Recruit = require "recruit"

function love.load(args)
    --love.window.setFullscreen( true )
    love.window.setMode( 800, 600, {fullscreen = false, vsync = true, fsaa = 0} )

    drawTimer = Timer.new()

    obstacles = {}
    objects = {}
    recruits = {}
    blueSprite = love.graphics.newImage('pics/GathererSpriteSheet.png')
    redSprite = love.graphics.newImage('pics/HunterSpriteSheet.png')
    spriteGrid = anim8.newGrid(32, 19, blueSprite:getWidth(), blueSprite:getHeight())

    arrowSprite = love.graphics.newImage('pics/ArrowSprite.png')
    arrowGrid = anim8.newGrid(16, 16, arrowSprite:getWidth(), arrowSprite:getHeight())

    enemySprite = love.graphics.newImage('pics/EnemySprite.png')
    enemyGrid = anim8.newGrid(32, 32, enemySprite:getWidth(), enemySprite:getHeight())

    hitSprite = love.graphics.newImage('pics/Hit Effect.png')
    hitGrid = anim8.newGrid(64, 64, hitSprite:getWidth(), hitSprite:getHeight())

    loader.path = "maps/"

    map = loader.load("Map.tmx")

    enemies = {}
    allFoodPlaces = {}

    parseMapSpecials(map("Enemies"))
    parseMapSpecials(map("Berries"))
    parseMapSpecials(map("Trees"))
    parseMapSpecials(map("Recruits"))

    nomads = Group()
    local start = vector(4096,8192)
    local around = vector(400,300)
    for i=1,2 do
        local dx = love.math.random(-400,400 )
        local dy = love.math.random(-400,400 )
        local v = vector(dx,dy)
        nomads:addMember(Blue(start+v,10))
    end

    for i=1,3 do
        local dx = love.math.random(-400,400 )
        local dy = love.math.random(-400,400 )
        local v = vector(dx,dy)
        nomads:addMember(Red(start+v,10))
    end

    for i, actor in ipairs(nomads.member) do
        actor.goal = start
    end

    table.insert(recruits, Recruit(vector(700,400),10,5,"blue"))

    player = Player()

    playerGroup = nomads

    camera = Camera(400, 300)
    camera:zoomTo(1)


    --sounds
    music = love.audio.newSource( "sounds/Music02.mp3" )
    music:setLooping(true)
    music:setVolume( 0.5 )
    music:play()

    dieS = love.audio.newSource( "sounds/SFX/Die.wav", "static" )
    eatS = love.audio.newSource( "sounds/SFX/Eat.wav", "static" )
    gatherS = love.audio.newSource( "sounds/SFX/Gather.wav", "static" )
    hitS = love.audio.newSource( "sounds/SFX/Hit.wav", "static" )
    leveldownS = love.audio.newSource( "sounds/SFX/LevelDown.wav", "static" )
    levelupS = love.audio.newSource( "sounds/SFX/LevelUp.wav", "static" )
    recruitS = love.audio.newSource( "sounds/SFX/Recruit.wav", "static" )
end

function love.update(dt)
    Timer.update(dt)

    nomads:update(dt)
    local m = nomads:middle()
    camera:lookAt(m.x,m.y)
    map:setDrawRange(m.x-400,m.y-300,800,600)

    player:update(dt)

    for i, e in ipairs(enemies) do
        e:update(dt)
    end
    for i, r in ipairs(recruits) do
        r:update(dt)
    end
end

function love.draw()
    camera:attach()

    map:draw()

    for i, place in ipairs(allFoodPlaces) do
        --place:draw()
    end

    for i, r in ipairs(recruits) do
        r:draw()
    end

    nomads:draw()
    --local m = nomads:middle()
    --love.graphics.circle("fill",m.x,m.y, 10, 100)

    for i, obstacle in ipairs(obstacles) do
        --obstacle:draw()
    end
    --love.graphics.print("Hello World", 400, 300)

    --love.graphics.circle( "fill", 300, 300, 50, 100 )

    for i, e in ipairs(enemies) do
        e:draw()
    end

    drawTimer:update(love.timer.getDelta( ))

    camera:detach()

    love.graphics.print( "food: " .. playerGroup.food, 10, 10, 0, 3, 3)
end

function love.keypressed( key, isrepeat )
    if key == "escape" then
        love.event.quit()
    end

    if key == "r" then
        music:stop()
        love.load()
    elseif key == "f" then
        local fullscreen = love.window.getFullscreen( )
        love.window.setFullscreen( not fullscreen )
    end
end

function love.mousepressed(x, y, button)
    x, y = camera:worldCoords(x,y)
    if button == "l" then
        for i, actor in ipairs(playerGroup.member) do
            actor.goal = vector(x,y)
        end
    elseif button == "r" then
        player:focusAt(vector(x,y))
    -- elseif button =="m" then
    --     player:focusAt(vector(x,y))
    --     if player.focus then
    --         player.focus:changeJob()
    --     end
    end
end

function parseMapSpecials(layer)
    for x,y, tile in layer:iterate() do
        local x,y = x*tile.width, y*tile.height
        if tile.properties.solid == 1 then
            --print(PrettyPrint(tile))
            table.insert( obstacles, Object(vector(x + tile.width/2, y + tile.width/2), tile.width/2) )
        end
        if tile.properties.enemy == 1 then
            table.insert(enemies, Enemy(vector(x, y),50,30))
        end
        if tile.properties.food == 1 then
            table.insert(allFoodPlaces, FoodPlace(vector(x + tile.width/2,y + tile.width/2 ),100,100))
        end
        if tile.properties.recruit then
            table.insert(recruits, Recruit(vector(x,y),10,tile.properties.recruit))
        end
    end
end