vector = require "hump.vector"


function filter( table, f )
    local filtered = {}
    for i, v in pairs(table) do
        if f(v) then table.insert(filtered, v) end
    end
end

function wihtoutTeam( heroTable, team )
    local function notFromTeam( hero )
        return hero.team ~= team
    end
    filter(heroTable, notFromTeam)
end

function empty(table)
    return next(table) == nil
end


function pointInImage(point, image, imagepos)
    return inRect(imagepos.x, imagepos.y, image:getWitdh(), image:getHeight(), point.x, point.y)
end


function inRect (x0, y0, w, h, x, y)
  return x > x0 and x < x0+w and y > y0 and y < y0+h
end

function inRect2 (point,corner,v1,v2)
  local v = point - corner
  local a = v*v1
  local b = v1*v1
  local c = v*v2
  local d = v2*v2
  return 0<=a and a<=b and 0<=c and c<=d
end



function filterIterator( iterator, f )

     local function filtered()
        for v in iterator() do
            if f(v) then
                coroutine.yield(v)
            end
        end
    end

    local filteredCo = coroutine.create(filtered)

    return function()
        local _, res = coroutine.resume(filteredCo)
        return res
    end
end

function table_invert(t)
  local u = { }
  for k, v in pairs(t) do u[v] = k end
  return u
end

--checks if two circles collide
function collide(pos1,r1,pos2,r2)
    return pos1:dist(pos2) < r1 + r2
end

function angleToDirection(angle)
  if angle >= 45 and angle <= 135 then
    return "up"
  elseif angle >= 135 and angle <= 225 then
    return "left"
  elseif angle >= 225 and angle <= 315 then
    return "down"
  elseif angle >= 315 and angle <= 45 then
    return "right"
  end
end

function radianToDirection(radian)
  local radian = radian % (2*math.pi)
  if radian >= 0.785398163 and radian <= 2.35619449 then
    return "up"
  elseif radian >= 2.35619449 and radian <= 3.926990817 then
    return "left"
  elseif radian >= 3.926990817 and radian <= 5.497787144 then
    return "down"
  elseif radian >= 5.497787144 and radian <= 0.785398163 then
    return "right"
  end
end

function table.removeV(t, value)
  for i, v in ipairs(t) do
    if v == value then table.remove(t, i) end
  end
end